﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.BusinessLogicsContracts;
using Microsoft.Extensions.Logging;

namespace FlowerShop
{
    public partial class FormBouquets : Form
    {
        private readonly ILogger _logger;
        private readonly IBouquetLogic _logic;

        public FormBouquets(ILogger<FormBouquets> logger, IBouquetLogic logic)
        {
            InitializeComponent();
            _logger = logger;
            _logic = logic;
        }

        private void LoadData()
        {
            try
            {
                var list = _logic.ReadList(null);

                if (list != null)
                {
                    dataGridView.DataSource = list;
                    dataGridView.Columns["Id"].Visible = false;
                    dataGridView.Columns["BouquetName"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
                    dataGridView.Columns["BouquetComponents"].Visible = false;
                }

                _logger.LogInformation("Receiving bouquets");

            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error during receiving bouquets");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void AddButton_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormBouquet));

            if (service is FormBouquet form)
            {
                if (form.ShowDialog() == DialogResult.OK)
                {
                    LoadData();
                }
            }
        }

        private void ChangeButton_Click(object sender, EventArgs e)
        {
            if (dataGridView.SelectedRows.Count == 1)
            {
                var service = Program.ServiceProvider?.GetService(typeof(FormBouquet));

                if (service is FormBouquet form)
                {
                    form.Id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);

                    if (form.ShowDialog() == DialogResult.OK)
                    {
                        LoadData();
                    }
                }
            }
        }

        private void DeleteButton_Click(object sender, EventArgs e)
        {
            if (dataGridView.SelectedRows.Count == 1)
            {
                if (MessageBox.Show("Удалить запись?", "Вопрос", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
                    _logger.LogInformation("Deleting bouquet");

                    try
                    {
                        if (!_logic.Delete(new BouquetBindingModel { Id = id }))
                        {
                            throw new Exception("Error during deleting bouquet");
                        }

                        LoadData();
                    }
                    catch (Exception ex)
                    {
                        _logger.LogError(ex, "Error deleting bouquet");
                        MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }

        private void UpdateButton_Click(object sender, EventArgs e)
        {
            LoadData();
        }

        private void BouquetsForm_Load(object sender, EventArgs e)
        {
            LoadData();
        }
    }
}
