﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.BusinessLogicsContracts;
using Microsoft.Extensions.Logging;

namespace FlowerShop
{
    public partial class FormShopSell : Form
    {
        private readonly ILogger _logger;
        private readonly IBouquetLogic _logicBouquet;
        private readonly IShopLogic _logicShop;

        public FormShopSell(ILogger<FormShopSell> logger, IBouquetLogic logicBouquet, IShopLogic logicShop)
        {
            InitializeComponent();
            _logger = logger;
            _logicBouquet = logicBouquet;
            _logicShop = logicShop;
        }

        private void FormShopSell_Load(object sender, EventArgs e)
        {
            _logger.LogInformation("Loading bouquets for selling");

            try
            {
                var list = _logicBouquet.ReadList(null);
                if (list != null)
                {
                    comboBoxBouquet.DisplayMember = "BouquetName";
                    comboBoxBouquet.ValueMember = "Id";
                    comboBoxBouquet.DataSource = list;
                    comboBoxBouquet.SelectedItem = null;
                }

            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error during loading bouquets for selling");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonSell_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxCount.Text))
            {
                MessageBox.Show("Заполните поле Количество", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (comboBoxBouquet.SelectedValue == null)
            {
                MessageBox.Show("Выберите букет", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            _logger.LogInformation("Создание продажи");

            try
            {
                var operationResult = _logicShop.SellBouquet(
                    new BouquetBindingModel
                    {
                        Id = Convert.ToInt32(comboBoxBouquet.SelectedValue)
                    },
                    Convert.ToInt32(textBoxCount.Text)
                );

                if (!operationResult)
                {
                    throw new Exception("Ошибка при создании продажи. Дополнительная информация в логах.");
                }

                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);

                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания продажи");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
