﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.BusinessLogicsContracts;
using FlowerShopContracts.SearchModels;
using Microsoft.Extensions.Logging;


namespace FlowerShop
{
    public partial class FormShopSupply : Form
    {
        private readonly ILogger _logger;
        private readonly IBouquetLogic _logicBouquet;
        private readonly IShopLogic _logicShop;

        public FormShopSupply(ILogger<FormShopSupply> logger, IBouquetLogic logicBouquet, IShopLogic logicShop)
        {
            InitializeComponent();
            _logger = logger;
            _logicBouquet = logicBouquet;
            _logicShop = logicShop;
        }

        private void FormShopSupply_Load(object sender, EventArgs e)
        {
            _logger.LogInformation("Loading bouquets for supplying");

            try
            {
                var list = _logicBouquet.ReadList(null);
                if (list != null)
                {
                    comboBoxBouquet.DisplayMember = "BouquetName";
                    comboBoxBouquet.ValueMember = "Id";
                    comboBoxBouquet.DataSource = list;
                    comboBoxBouquet.SelectedItem = null;
                }

            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error during loading bouquets for supplying");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            _logger.LogInformation("Loading shops for supplying");

            try
            {
                var list = _logicShop.ReadList(null);
                if (list != null)
                {
                    comboBoxShop.DisplayMember = "Name";
                    comboBoxShop.ValueMember = "Id";
                    comboBoxShop.DataSource = list;
                    comboBoxShop.SelectedItem = null;
                }

            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error during loading shops for supplying");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxCount.Text))
            {
                MessageBox.Show("Заполните поле Количество", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (comboBoxBouquet.SelectedValue == null)
            {
                MessageBox.Show("Выберите букет", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (comboBoxShop.SelectedValue== null)
            {
                MessageBox.Show("Выберите магазин", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            _logger.LogInformation("Creation of supply");

            try
            {
                var operationResult = _logicShop.SupplyBouquets(
                    new ShopSearchModel { Id = Convert.ToInt32(comboBoxShop.SelectedValue), Name = comboBoxShop.Text },
                    new BouquetBindingModel { Id = Convert.ToInt32(comboBoxBouquet.SelectedValue), BouquetName = comboBoxBouquet.Text },
                    Convert.ToInt32(textBoxCount.Text)
                );

                if (!operationResult)
                {
                    throw new Exception("Ошибка при создании поставки. Дополнительная информация в логах.");
                }

                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error during creation of supply");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
