﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.SearchModels;
using FlowerShopContracts.StoragesContracts;
using FlowerShopContracts.ViewModels;
using FlowerShopFileImplement.Models;

namespace FlowerShopFileImplement.Implements
{
    public class BouquetStorage : IBouquetStorage
    {
        private readonly DataFileSingleton source;

        public BouquetStorage()
        {
            source = DataFileSingleton.GetInstance();
        }

        public BouquetViewModel? GetElement(BouquetSearchModel model)
        {
            if (string.IsNullOrEmpty(model.BouquetName) && !model.Id.HasValue)
            {
                return null;
            }
            return source.Bouquets.FirstOrDefault(x => (!string.IsNullOrEmpty(model.BouquetName) && x.BouquetName == model.BouquetName) || (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }

        public List<BouquetViewModel> GetFilteredList(BouquetSearchModel model)
        {
            if (string.IsNullOrEmpty(model.BouquetName))
            {
                return new();
            }
            return source.Bouquets.Where(x => x.BouquetName.Contains(model.BouquetName)).Select(x => x.GetViewModel).ToList();
        }

        public List<BouquetViewModel> GetFullList()
        {
            return source.Bouquets.Select(x => x.GetViewModel).ToList();
        }

        public BouquetViewModel? Insert(BouquetBindingModel model)
        {
            model.Id = source.Bouquets.Count > 0 ? source.Bouquets.Max(x => x.Id) + 1 : 1;
            
            var newBouquet = Bouquet.Create(model);
            if (newBouquet == null)
            {
                return null;
            }

            source.Bouquets.Add(newBouquet);
            source.SaveBouquets();

            return newBouquet.GetViewModel;
        }

        public BouquetViewModel? Update(BouquetBindingModel model)
        {
            var bouquet = source.Bouquets.FirstOrDefault(x => x.Id == model.Id);
            if (bouquet == null)
            {
                return null;
            }

            bouquet.Update(model);
            source.SaveBouquets();

            return bouquet.GetViewModel;
        }

        public BouquetViewModel? Delete(BouquetBindingModel model)
        {
            var bouquet = source.Bouquets.FirstOrDefault(x => x.Id == model.Id);
            if (bouquet == null)
            {
                return null;
            }

            source.Bouquets.Remove(bouquet);
            source.SaveBouquets();

            return bouquet.GetViewModel;
        }
    }
}
