﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.SearchModels;
using FlowerShopContracts.StoragesContracts;
using FlowerShopContracts.ViewModels;
using FlowerShopFileImplement.Models;
using System.Linq;

namespace FlowerShopFileImplement.Implements
{
    public class ComponentStorage : IComponentStorage
    {
        private readonly DataFileSingleton source;

        public ComponentStorage()
        {
            source = DataFileSingleton.GetInstance();
        }

        public ComponentViewModel? GetElement(ComponentSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ComponentName) && !model.Id.HasValue)
            {
                return null;
            }

            return source.Components.FirstOrDefault(x => (!string.IsNullOrEmpty(model.ComponentName) && x.ComponentName == model.ComponentName) || (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }

        public List<ComponentViewModel> GetFilteredList(ComponentSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ComponentName))
            {
                return new();
            }

            return source.Components.Where(x => x.ComponentName.Contains(model.ComponentName)).Select(x => x.GetViewModel).ToList();
        }

        public List<ComponentViewModel> GetFullList()
        {
            return source.Components.Select(x => x.GetViewModel).ToList();
        }

        public ComponentViewModel? Insert(ComponentBindingModel model)
        {
            model.Id = source.Components.Count > 0 ? source.Components.Max(x => x.Id) + 1 : 1;
            
            var newComponent = Component.Create(model);
            if (newComponent == null)
            {
                return null;
            }

            source.Components.Add(newComponent);
            source.SaveComponents();

            return newComponent.GetViewModel;
        }

        public ComponentViewModel? Update(ComponentBindingModel model)
        {
            var component = source.Components.FirstOrDefault(x => x.Id == model.Id);
            if (component == null)
            {
                return null;
            }

            component.Update(model);
            source.SaveComponents();

            return component.GetViewModel;
        }

        public ComponentViewModel? Delete(ComponentBindingModel model)
        {
            var component = source.Components.FirstOrDefault(x => x.Id == model.Id);
            if (component == null)
            {
                return null;
            }

            source.Components.Remove(component);
            source.SaveComponents();

            return component.GetViewModel;
        }
    }
}
