﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.SearchModels;
using FlowerShopContracts.StoragesContracts;
using FlowerShopContracts.ViewModels;
using FlowerShopFileImplement.Models;

namespace FlowerShopFileImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        private readonly DataFileSingleton source;

        public OrderStorage()
        {
            source = DataFileSingleton.GetInstance();
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }

            return AttachBouquetName(source.Orders.FirstOrDefault(x => (model.Id.HasValue && x.Id == model.Id))?.GetViewModel);
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return new();
            }

            return source.Orders.Where(x => x.Id == model.Id).Select(x => AttachBouquetName(x.GetViewModel)).ToList();
        }

        public List<OrderViewModel> GetFullList()
        {
            return source.Orders.Select(x => AttachBouquetName(x.GetViewModel)).ToList();
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            var order = source.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }

            order.Update(model);
            source.SaveOrders();

            return AttachBouquetName(order.GetViewModel);
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            model.Id = source.Orders.Count > 0 ? source.Orders.Max(x => x.Id) + 1 : 1;
            var newOrder = Order.Create(model);

            if (newOrder == null)
            {
                return null;
            }

            source.Orders.Add(newOrder);
            source.SaveOrders();

            return AttachBouquetName(newOrder.GetViewModel);
        }

        public OrderViewModel? Delete(OrderBindingModel model)
        {
            var order = source.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }

            source.Orders.Remove(order);
            source.SaveOrders();

            return AttachBouquetName(order.GetViewModel);
        }

        private OrderViewModel AttachBouquetName(OrderViewModel? model)
        {
            if (model == null)
            {
                return null;
            }

            var bouquet = source.Bouquets.Find(b => b.Id == model.BouquetId);
            model.BouquetName = bouquet is null ? String.Empty : bouquet.BouquetName;

            return model;
        }
    }
}
