﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.SearchModels;
using FlowerShopContracts.StoragesContracts;
using FlowerShopContracts.ViewModels;
using FlowerShopDataModels.Models;
using FlowerShopFileImplement.Models;

namespace FlowerShopFileImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        private readonly DataFileSingleton source;

        public ShopStorage()
        {
            source = DataFileSingleton.GetInstance();
        }

        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }

            return source.Shops.FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)?.GetViewModel;
        }

        public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name))
            {
                return new();
            }

            return source.Shops.Select(x => x.GetViewModel).Where(x => x.Name.Contains(model.Name ?? string.Empty)).ToList();
        }

        public List<ShopViewModel> GetFullList()
        {
            return source.Shops.Select(shop => shop.GetViewModel).ToList();
        }

        public ShopViewModel? Insert(ShopBindingModel model)
        {
            model.Id = source.Shops.Count > 0 ? source.Shops.Max(x => x.Id) + 1 : 1;
            
            var newShop = Shop.Create(model);
            if (newShop == null)
            {
                return null;
            }

            source.Shops.Add(newShop);
            source.SaveShops();

            return newShop.GetViewModel;
        }

        public ShopViewModel? Update(ShopBindingModel model)
        {
            var shop = source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (shop == null)
            {
                return null;
            }

            shop.Update(model);
            source.SaveShops();

            return shop.GetViewModel;
        }

        public ShopViewModel? Delete(ShopBindingModel model)
        {
            var shop = source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (shop == null)
            {
                return null;
            }

            source.Shops.Remove(shop);
            source.SaveShops();

            return shop.GetViewModel;
        }

        public bool SellBouquet(IBouquetModel model, int count)
        {
            var bouquet = source.Bouquets.FirstOrDefault(x => x.Id == model.Id);

            var countStore = count;

            if (bouquet == null)
            {
                return false;
            }

            foreach (var shop in source.Shops)
            {
                foreach (var bouq in shop.ShopBouquets)
                {
                    if (bouq.Value.Item1.Id == bouquet.Id)
                    {
                        count -= bouq.Value.Item2;
                    }
                    if (count <= 0)
                    {
                        break;
                    }
                }
            }

            if (count > 0)
            {
                return false;
            }

            count = countStore;

            foreach(var shop in source.Shops)
            {
                var bouquets = shop.ShopBouquets;

                foreach (var bouq in bouquets.Where(x => x.Value.Item1.Id == bouquet.Id))
                {
                    var min = Math.Min(bouq.Value.Item2, count);
                    bouquets[bouq.Value.Item1.Id] = (bouq.Value.Item1, bouq.Value.Item2 - min);
                    count -= min;

                    if (count <= 0)
                    {
                        break;
                    }
                }

                shop.Update(new ShopBindingModel
                {
                    Id = shop.Id,
                    Name = shop.Name,
                    Address = shop.Address,
                    OpeningDate = shop.OpeningDate,
                    MaxCountBouquets = shop.MaxCountBouquets,
                    ShopBouquets = bouquets
                });
                source.SaveShops();
            }

            if (count > 0)
            {
                return false;
            }
            return true;
        }
    }
}
