﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.SearchModels;
using FlowerShopContracts.StoragesContracts;
using FlowerShopContracts.ViewModels;
using FlowerShopListImplement.Models;

namespace FlowerShopListImplement.Implements
{
    public class BouquetStorage : IBouquetStorage
    {
        private readonly DataListSingleton _source;

        public BouquetStorage()
        {
            _source = DataListSingleton.GetInstance();
        }

        public List<BouquetViewModel> GetFullList()
        {
            var result = new List<BouquetViewModel>();

            foreach (var bouquet in _source.Bouquets)
            {
                result.Add(bouquet.GetViewModel);
            }

            return result;
        }

        public List<BouquetViewModel> GetFilteredList(BouquetSearchModel model)
        {
            var result = new List<BouquetViewModel>();

            if (string.IsNullOrEmpty(model.BouquetName))
            {
                return result;
            }

            foreach (var bouquet in _source.Bouquets)
            {
                if (bouquet.BouquetName.Contains(model.BouquetName))
                {
                    result.Add(bouquet.GetViewModel);
                }
            }

            return result;
        }

        public BouquetViewModel? GetElement(BouquetSearchModel model)
        {
            if (string.IsNullOrEmpty(model.BouquetName) && !model.Id.HasValue)
            {
                return null;
            }

            foreach (var bouquet in _source.Bouquets)
            {
                if ((!string.IsNullOrEmpty(model.BouquetName) && bouquet.BouquetName == model.BouquetName) || (model.Id.HasValue && bouquet.Id == model.Id))
                {
                    return bouquet.GetViewModel;
                }
            }

            return null;
        }

        public BouquetViewModel? Insert(BouquetBindingModel model)
        {
            model.Id = 1;
            foreach (var bouquet in _source.Bouquets)
            {
                if (model.Id <= bouquet.Id)
                {
                    model.Id = bouquet.Id + 1;
                }
            }

            var newBouquet = Bouquet.Create(model);
            if (newBouquet == null)
            {
                return null;
            }

            _source.Bouquets.Add(newBouquet);

            return newBouquet.GetViewModel;
        }

        public BouquetViewModel? Update(BouquetBindingModel model)
        {
            foreach (var bouquet in _source.Bouquets)
            {
                if (bouquet.Id == model.Id)
                {
                    bouquet.Update(model);
                    return bouquet.GetViewModel;
                }
            }

            return null;
        }

        public BouquetViewModel? Delete(BouquetBindingModel model)
        {
            for (int i = 0; i < _source.Bouquets.Count; ++i)
            {
                if (_source.Bouquets[i].Id == model.Id)
                {
                    var element = _source.Bouquets[i];
                    _source.Bouquets.RemoveAt(i);
                    return element.GetViewModel;
                }
            }

            return null;
        }
    }
}