﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.ViewModels;
using FlowerShopDataModels.Models;

namespace FlowerShopListImplement.Models
{
    public class Shop : IShopModel
    {
        public int Id { get; private set; }

        public string Name { get; private set; } = string.Empty;

        public string Address { get; private set; } = string.Empty;

        public DateTime OpeningDate { get; private set; }

        public int MaxCountBouquets { get; private set; }

        public Dictionary<int, (IBouquetModel, int)> ShopBouquets { get; private set; } = new();

        public static Shop? Create(ShopBindingModel model)
        {
            if (model == null)
            {
                return null;
            }

            return new Shop()
            {
                Id = model.Id,
                Name = model.Name,
                Address = model.Address,
                OpeningDate = model.OpeningDate,
                MaxCountBouquets = model.MaxCountBouquets,
                ShopBouquets = new()
            };
        }

        public void Update(ShopBindingModel? model)
        {
            if (model == null)
            {
                return;
            }

            Name = model.Name;
            Address = model.Address;
            ShopBouquets = model.ShopBouquets;
            MaxCountBouquets = model.MaxCountBouquets;
        }

        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            Name = Name,
            Address = Address,
            OpeningDate = OpeningDate,
            ShopBouquets = ShopBouquets,
            MaxCountBouquets = MaxCountBouquets
        };
    }
}
